

 /*
 ************************************
 ************************************
 FARMER FOX CHICKEN CORN PUZZLE
 ************************************
 ************************************

 File: FarmerFoxControl16
 Date: 6/16/21
 Written by: David Goodsell, Apple Valley, CA 

 This program supports the Farmer,Fox,Chicken,Corn puzzle published in Nuts and Volts magazine.
 It was written for the Arduino Nano Every microcontroller.
 The Arduino Nano Every has 22 digital I/O pins
 The system uses 20 pins plus a 3 to 8 decoder to allow more sound trigger pins.
 The wav trigger sound board has 16 hard-wired inputs that can be pulse triggered or run continuously. 14 are used.
 The sound board can also be triggered through a serial port, but it was not used for this project.
 
 */

 const int farmerSouth =  18;
 const int foxSouth =     19;
 const int chickenSouth = 20;
 const int cornSouth =    21;
 
 const int farmerNorth =  14;
 const int foxNorth =     15;
 const int chickenNorth = 16;
 const int cornNorth =    17;
 
 const int swSouth =       8;
 const int motorRunning =  9;
 const int swNorth =      10;
   
 const int wav001 =       7;  
 const int wav002 =       6;
 const int wav003 =       5;
 const int wav004 =       4;
 const int wav005 =       3;   //not used, hardwired instead
 const int wav006 =       2;
 
 //const int wav007 =     x;  //not used, see below explanation
 //const int wav008 =     x;  //not used
 //wav007 and wav008 are not used but must be included in wav board INIT file and pins are grounded through 1K resistors
 
 const int beeper =            12;
 
 const int LS138_A0 =          13;   //the following 3 pins trigger 8 wav sounds, wav009 to wav016 
 const int LS138_A1 =          0;    //notice that A1 is pin 1 and A2 is zero.
                                     //The pins on the Arduino are not numbered in digital/physical sequence. It goes, 1,0,2,3,4,5....
 const int LS138_A2 =          1; 
 const int LS138_E2 =          11;

int x;
int y;
int z;
int a;

int southStatus;
int northStatus;
int flagReadyToPlay;
int flagWinnerWinner;
int flagYouLoseBeginAgain;
int swSouthState;
int lastswSouthState;
int swNorthState;
int lastswNorthState;
int reading;
unsigned long lastDebounceTime = 0;
unsigned long debounceDelay = 50;


//***************************************************************************************

void setup() 
{ 
 pinMode(farmerSouth, INPUT);
 pinMode(foxSouth, INPUT);
 pinMode(chickenSouth, INPUT);
 pinMode(cornSouth, INPUT);
 
 pinMode(farmerNorth, INPUT);
 pinMode(foxNorth, INPUT);
 pinMode(chickenNorth, INPUT);
 pinMode(cornNorth, INPUT);

 pinMode(swSouth, INPUT);
 pinMode(swNorth, INPUT);
 pinMode(motorRunning, INPUT);

 pinMode(wav001, OUTPUT);
 pinMode(wav002, OUTPUT);     
 pinMode(wav003, OUTPUT);
 pinMode(wav004, OUTPUT);
 pinMode(wav005, OUTPUT);
 pinMode(wav006, OUTPUT);
 //pinMode(wav007, OUTPUT);
 //pinMode(wav008, OUTPUT);
 pinMode(beeper, OUTPUT); 

 pinMode(LS138_A0, OUTPUT); //wav009 to wav016     
 pinMode(LS138_A1, OUTPUT);   
 pinMode(LS138_A2, OUTPUT);    
 pinMode(LS138_E2, OUTPUT); //enable pulse       
 
 digitalWrite(LS138_A0, 1);
 digitalWrite(LS138_A1, 1);
 digitalWrite(LS138_A2, 1);
 digitalWrite(LS138_E2, 0);
 onebeep();

 //Serial.begin(9600);   //for displayLCD

 //ALLOW SOUND BOARD TO BOOT UP
 //****************************
 delay(2000);

 //INHIBIT WINNER WINNER MESSAGE if board is still set up for Winner
 //****************************************************************
  if (((digitalRead(chickenNorth)) == 1) && ((digitalRead(foxNorth)) == 1) &&
  ((digitalRead(cornNorth)) == 1) && ((digitalRead(farmerNorth)) == 1) && (digitalRead(swNorth)) == 0)
  {
  flagWinnerWinner = 1;    //flag inhibits WinnerWinner. Resets when all pieces are returned to their South spots(i.e. Ready To Play)
  }

//MESSAGE AND BOARD STATUS SETUP
//**********************************
playToBeginMessage();
updateNorthSouthLimitSwitchesState();

}   //Setup done

//*********************************************************************************************************************************
//*********************************************************************************************************************************
//                             START OF PROGRAM
//*********************************************************************************************************************************
//*********************************************************************************************************************************

void loop() {

 //Panic/Reset button   
 //*******************
 //Connected to Reset pin of Arduino


//Check motor running, play chicken clucking if on ferry
//************************************************************************
  if ((digitalRead(motorRunning)) == 0)
   {
   if (((digitalRead(chickenNorth)) == 0) && ((digitalRead(chickenSouth)) == 0)) (digitalWrite(wav006,1));  //clucking
   }
  else
   {  
   digitalWrite(wav006,0);  //clucking
   }


//Check for all pieces to be put in place in South including Ferry then play ReadyToPlay
//***************************************************************************************
  if (flagReadyToPlay == 0)
  {
  checkSouthStatus();
    if (((southStatus == 1111) && (((digitalRead(swSouth)) == 0))) == 1)  //four pieces in place and limit switch clicked
    {
    flagWinnerWinner = 0;  
    playReadyToPlayMesssage();
    flagReadyToPlay = 1; 
    }
  }
  
   
//Check South limit switch
//**************************************************************************************************
//Debounce South Limit Switch
//*********************************
reading = (digitalRead(swSouth));    
 if (reading != lastswSouthState)
  {        
    delay(50);
    if (reading != lastswSouthState)
     {
     swSouthState = reading;
     lastswSouthState = swSouthState;
     }
    else 
     {
     goto noChangeswSouth;   
     }
  }
 else
  {
  goto noChangeswSouth; 
  } 

//*************
//Just Left South Dock
//*************
if (swSouthState == 1)  
{
playDingDing();  
y = checkSouthStatus();
//1000 = farmer remains at South  go back
// 111 = farmer left all three    go back there's trouble"  trig 013  
// 110 = left fox and chicken     go back 
//  11 = left chicken and corn    go back 

if (y >= 1000) (y = 1000);   

switch(y)
 {
  case 11:
   {
   playMessage(0,0,0);  //go back chicken and corn
   delay(2000);
   break;
   }
  case 110:
   {
   playMessage(1,0,1);  //go back fox and chicken
   delay(2000);
   break; 
   }
  case 111:
   {
   playMessage(1,0,0);   //"go back there's trouble"  trig 013
   delay(2000);
   break;
   }
  case 1000:
   {
   playMessage(1,1,0);  //go back farmer must go along  (must be on ferry too)
   delay(2500);
   break; 
   }
 }
}

//******************
//Just Returned to South Dock
//******************
if (swSouthState == 0)
{
y = checkNorthStatus();
playDingDing();
//delay(2000);                                            
//flagReadyToPlay = 0;
// 1111 = all on North side      winner
// 111 = farmer left all three   YOU LOSE!  play wav
// 110 = left fox and chicken    YOU LOSE!  play wav
// 11 = left chicken and corn    YOU LOSE!  play wav

switch(y)
 {
  case 11:
   {
   playMessage(0,1,1);   //you lose, chicken ate corn
   delay(5000);
   //playToBeginMessage();
   loop();
   break;
   }
  case 110:
   {
   playMessage(0,1,0);  //you lose, fox ate chicken
   delay(5000);
   //playToBeginMessage();
   loop();
   break; 
   }
  case 111:
   {
   playMessage(0,0,1);  //you lose, things got eaten
   delay(5000);
   //playToBeginMessage();
   loop();
   break; 
   }
 }

}
noChangeswSouth:
//delay(500);


//Check North limit switch
//*******************************************************************************************
//Debounce North Limit Switch
//*********************************
reading = (digitalRead(swNorth));    
 if (reading != lastswNorthState)
  {        
    delay(50);
    if (reading != lastswNorthState)
     {
     swNorthState = reading;
     lastswNorthState = swNorthState;
     }
    else 
     {
     goto noChangeswNorth;   
     }
  }
 else
  {
  goto noChangeswNorth; 
  } 
//********************************

//*************
//Just Left North Dock
//*************
if (swNorthState == 1)  
{
y = checkNorthStatus();
playDingDing();
flagWinnerWinner = 0;   
// 111 = farmer left all three   go back
// 110 = left fox and chicken    go back wav002
// 11 = left chicken and corn    go back wav001

switch(y)
 {
  case 11:
   {
   playMessage(0,0,0);  //go back chicken and corn
   delay(2000);
   break;
   }
  case 110:
   {
   playMessage(1,0,1);  //go back fox and chicken
   delay(2000);
   break; 
   }
  case 111:
   {
   playMessage(1,0,0);   //go back there's trouble
   delay(2000);
   break; 
   }
 }
}

//******************
//Just Entered North Dock
//******************
if (swNorthState == 0)
{
y = checkSouthStatus();
playDingDing();
// 1111 = all on North side      winner
// 111 = farmer left all three   YOU LOSE!
// 110 = left fox and chicken    YOU LOSE!  play wav
// 11 = left chicken and corn    YOU LOSE!  play wav

switch(y)
 {
  case 11:
   {
   playMessage(0,1,1);   //you lose, chicken ate corn
   //flagYouLoseBeginAgain = 1;
   delay(5000);
   //playToBeginMessage();
   loop();
   break;
   }
  case 110:
   {
   playMessage(0,1,0);  //you lose, fox ate chicken
   delay(5000);
   //playToBeginMessage();
   loop();
   break; 
   }
  case 111:
   {
   playMessage(0,0,1);  //you lose, all are together
   delay(5000);
   //playToBeginMessage();
   loop();
   break; 
   }
 }

}
noChangeswNorth:


//Check for Winner
//**********************************************************************************************
if (((digitalRead(chickenNorth)) == 1) && ((digitalRead(foxNorth)) == 1) && ((digitalRead(cornNorth)) == 1) &&
   ((digitalRead(farmerNorth)) == 1) && (digitalRead(swNorth)) == 0)
{
if (flagWinnerWinner == 0)
 {   
 playWinnerWinnerMessage();
 flagWinnerWinner = 1;    //flag inhibits WinnerWinner so WinnerWinner does not repeat each time a piece is lifting off its winning North spot 
                          //Resets when all pieces are returned to their South spots(i.e. Ready To Play) 
 //playToBeginMessage();
 }
}


}    //the big loop done

//************************************************************************************************************************************************************
//************************************************************************************************************************************************************
//************************************************************************************************************************************************************
//                      FUNCTIONS
//************************************************************************************************************************************************************
//************************************************************************************************************************************************************
//************************************************************************************************************************************************************

void playWinnerWinnerMessage()
{
digitalWrite(wav001, 1);
delay(50);
digitalWrite(wav001, 0);
delay(7000);
}

void playToBeginMessage()
{
digitalWrite(wav002, 1);
delay(50);
digitalWrite(wav002, 0);
delay(5000);
flagReadyToPlay = 0;
updateNorthSouthLimitSwitchesState();
}

void playReadyToPlayMesssage()
{
 digitalWrite(wav003, 1);
 delay(50);
 digitalWrite(wav003, 0);
 delay(2000);
}

void playDingDing()
{
  digitalWrite(wav004, 1);
  delay(50);
  digitalWrite(wav004, 0);
}

void updateNorthSouthLimitSwitchesState()
{
 if ((digitalRead(swSouth)) == 0)
  lastswSouthState = 0;   //docked south
 else
  lastswSouthState = 1;   //not docked
  
 if ((digitalRead(swNorth)) == 0)
  lastswNorthState = 0;   //docked north
 else
  lastswNorthState = 1;   //not docked
}

int checkSouthStatus()
{
  southStatus = 0;
  if ((digitalRead(farmerSouth)) == 1) (southStatus = (southStatus + 1000));
  if ((digitalRead(foxSouth)) == 1) (southStatus = (southStatus + 100));
  if ((digitalRead(chickenSouth)) == 1) (southStatus = (southStatus + 10));
  if ((digitalRead(cornSouth)) == 1) (southStatus = (southStatus + 1)); 
  return southStatus;
}

int checkNorthStatus()
{
  northStatus = 0;
  if ((digitalRead(farmerNorth)) == 1) (northStatus = (northStatus + 1000));
  if ((digitalRead(foxNorth)) == 1) (northStatus = (northStatus + 100));
  if ((digitalRead(chickenNorth)) == 1) (northStatus = (northStatus + 10));
  if ((digitalRead(cornNorth)) == 1) (northStatus = (northStatus + 1)); 
  return northStatus;
}

void playMessage(int Bit2, int Bit1, int Bit0)
{
digitalWrite(LS138_A0, Bit0);
digitalWrite(LS138_A1, Bit1);
digitalWrite(LS138_A2, Bit2);
digitalWrite(LS138_E2, 1);    //positive enable pulse
delay(50);
digitalWrite(LS138_E2, 0);

//Qx  BIN   TRIG 
//0 = 000 = trig 009  negative output pulses (Q0-Q7)
//1 = 001 = trig 010
//2 = 010 = trig 011
//3 = 011 = trig 012
//4 = 100 = trig 013
//5 = 101 = trig 014
//5 = 110 = trig 015
//7 = 111 = trig 016(No Connection)  (game rules button)

}


void onebeep()
{
digitalWrite(beeper, 1);
delay(100);
digitalWrite(beeper, 0);
delay(100);
}


void twobeep()
{
onebeep();
onebeep();
}


void threebeep()
{
onebeep();
onebeep();
onebeep();
}


//*******************************************
//           END
//*******************************************
//
